'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _action_types = require('./action_types');

function merge(viewModelName, aggregateId, state) {
  return {
    type: _action_types.MERGE,
    viewModelName: viewModelName,
    aggregateId: aggregateId,
    state: state
  };
}

function sendCommand(_ref) {
  var command = _ref.command,
      aggregateId = _ref.aggregateId,
      aggregateName = _ref.aggregateName,
      payload = _ref.payload;

  return {
    type: _action_types.SEND_COMMAND,
    command: command,
    aggregateId: aggregateId,
    aggregateName: aggregateName,
    payload: payload
  };
}

function subscribeViewmodel(viewModelName, aggregateId) {
  return {
    type: _action_types.SUBSCRIBE_VIEWMODEL,
    viewModelName: viewModelName,
    aggregateId: aggregateId
  };
}

function unsubscribeViewmodel(viewModelName, aggregateId) {
  return {
    type: _action_types.UNSUBSCRIBE_VIEWMODEL,
    viewModelName: viewModelName,
    aggregateId: aggregateId
  };
}

function subscribeReadmodel(readModelName, resolverName, variables, isReactive) {
  return {
    type: _action_types.SUBSCRIBE_READMODEL,
    readModelName: readModelName,
    resolverName: resolverName,
    variables: variables,
    isReactive: isReactive
  };
}

function unsubscribeReadmodel(readModelName, resolverName) {
  return {
    type: _action_types.UNSUBSCRIBE_READMODEL,
    readModelName: readModelName,
    resolverName: resolverName
  };
}

function loadReadmodelInitialState(readModelName, resolverName, initialState, serialId) {
  return {
    type: _action_types.READMODEL_LOAD_INITIAL_STATE,
    readModelName: readModelName,
    resolverName: resolverName,
    initialState: initialState,
    serialId: serialId
  };
}

function dropReadmodelState(readModelName, resolverName) {
  return {
    type: _action_types.READMODEL_DROP_STATE,
    readModelName: readModelName,
    resolverName: resolverName
  };
}

function provideViewModels(viewModels) {
  return {
    type: _action_types.PROVIDE_VIEW_MODELS,
    viewModels: viewModels
  };
}

function disconnect(reason) {
  return {
    type: _action_types.DISCONNECT,
    reason: reason
  };
}

exports.default = {
  merge: merge,
  sendCommand: sendCommand,
  subscribeViewmodel: subscribeViewmodel,
  unsubscribeViewmodel: unsubscribeViewmodel,
  provideViewModels: provideViewModels,
  subscribeReadmodel: subscribeReadmodel,
  unsubscribeReadmodel: unsubscribeReadmodel,
  loadReadmodelInitialState: loadReadmodelInitialState,
  dropReadmodelState: dropReadmodelState,
  disconnect: disconnect
};