'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.mockSubscribeAdapter = exports.subscribeViewmodel = undefined;

var subscribeViewmodel = exports.subscribeViewmodel = function () {
  var _ref2 = _asyncToGenerator( /*#__PURE__*/regeneratorRuntime.mark(function _callee(store, subscribeAdapter, viewModels, subscribers, requests, action) {
    var viewModelName, aggregateId, needChange, key, rawState, state;
    return regeneratorRuntime.wrap(function _callee$(_context) {
      while (1) {
        switch (_context.prev = _context.next) {
          case 0:
            viewModelName = action.viewModelName, aggregateId = action.aggregateId;
            needChange = !subscribers.viewModels[viewModelName] || !subscribers.aggregateIds[aggregateId];


            subscribers.viewModels[viewModelName] = (subscribers.viewModels[viewModelName] || 0) + 1;
            subscribers.aggregateIds[aggregateId] = (subscribers.aggregateIds[aggregateId] || 0) + 1;

            if (!needChange) {
              _context.next = 12;
              break;
            }

            key = (0, _util.getKey)(viewModelName, aggregateId);

            requests[key] = true;

            _context.next = 9;
            return (0, _load_initial_state2.default)(viewModelName, aggregateId);

          case 9:
            rawState = _context.sent;
            state = viewModels.find(function (_ref3) {
              var name = _ref3.name;
              return name === viewModelName;
            }).deserializeState(rawState);


            if (requests[key]) {
              delete requests[key];

              store.dispatch(_actions2.default.merge(viewModelName, aggregateId, state));

              subscribeAdapter.setSubscription({
                types: getEventTypes(viewModels, subscribers),
                aggregateIds: getAggregateIds(viewModels, subscribers)
              });
            }

          case 12:
          case 'end':
            return _context.stop();
        }
      }
    }, _callee, this);
  }));

  return function subscribeViewmodel(_x, _x2, _x3, _x4, _x5, _x6) {
    return _ref2.apply(this, arguments);
  };
}();

exports.getEventTypes = getEventTypes;
exports.getAggregateIds = getAggregateIds;
exports.unsubscribeViewmodel = unsubscribeViewmodel;
exports.subscribeReadmodel = subscribeReadmodel;
exports.unsubscribeReadmodel = unsubscribeReadmodel;
exports.createOrderedFetch = createOrderedFetch;
exports.createResolveMiddleware = createResolveMiddleware;

var _action_types = require('./action_types');

var _actions = require('./actions');

var _actions2 = _interopRequireDefault(_actions);

var _subscribe_adapter = require('./subscribe_adapter');

var _subscribe_adapter2 = _interopRequireDefault(_subscribe_adapter);

var _send_command = require('./send_command');

var _send_command2 = _interopRequireDefault(_send_command);

var _load_initial_state = require('./load_initial_state');

var _load_initial_state2 = _interopRequireDefault(_load_initial_state);

var _util = require('./util');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _asyncToGenerator(fn) { return function () { var gen = fn.apply(this, arguments); return new Promise(function (resolve, reject) { function step(key, arg) { try { var info = gen[key](arg); var value = info.value; } catch (error) { reject(error); return; } if (info.done) { resolve(value); } else { return Promise.resolve(value).then(function (value) { step("next", value); }, function (err) { step("throw", err); }); } } return step("next"); }); }; }

function _objectWithoutProperties(obj, keys) { var target = {}; for (var i in obj) { if (keys.indexOf(i) >= 0) continue; if (!Object.prototype.hasOwnProperty.call(obj, i)) continue; target[i] = obj[i]; } return target; }

var REFRESH_TIMEOUT = 1000;

function getEventTypes(viewModels, subscribers) {
  var eventTypes = {};

  Object.keys(subscribers.viewModels).forEach(function (viewModelName) {
    if (!subscribers.viewModels[viewModelName]) {
      return;
    }

    var _viewModels$find$proj = viewModels.find(function (_ref) {
      var name = _ref.name;
      return name === viewModelName;
    }).projection,
        Init = _viewModels$find$proj.Init,
        projection = _objectWithoutProperties(_viewModels$find$proj, ['Init']);

    Object.keys(projection).forEach(function (eventType) {
      eventTypes[eventType] = true;
    });
  });

  return Object.keys(eventTypes);
}

function getAggregateIds(viewModels, subscribers) {
  if (subscribers.aggregateIds['*'] > 0) {
    return '*';
  }

  return Object.keys(subscribers.aggregateIds).filter(function (aggregateId) {
    return subscribers.aggregateIds[aggregateId];
  });
}

function unsubscribeViewmodel(store, subscribeAdapter, viewModels, subscribers, requests, action) {
  var viewModelName = action.viewModelName,
      aggregateId = action.aggregateId;


  subscribers.viewModels[viewModelName] = Math.max((subscribers.viewModels[viewModelName] || 0) - 1, 0);
  subscribers.aggregateIds[aggregateId] = Math.max((subscribers.aggregateIds[aggregateId] || 0) - 1, 0);

  var needChange = !subscribers.viewModels[viewModelName] || !subscribers.aggregateIds[aggregateId];

  var key = (0, _util.getKey)(viewModelName, aggregateId);
  delete requests[key];

  if (needChange) {
    subscribeAdapter.setSubscription({
      types: getEventTypes(viewModels, subscribers),
      aggregateIds: getAggregateIds(viewModels, subscribers)
    });
  }
}

function subscribeReadmodel(store, readModelSubscriptions, subscribeAdapter, orderedFetch, action) {
  var _this = this;

  var readModelName = action.readModelName,
      resolverName = action.resolverName,
      variables = action.variables,
      isReactive = action.isReactive;

  var subscriptionKey = readModelName + ':' + resolverName;
  if (readModelSubscriptions.hasOwnProperty(subscriptionKey)) return;

  var fetchReadModel = function fetchReadModel() {
    if (!readModelSubscriptions.hasOwnProperty(subscriptionKey)) return;
    var checkSelfPromise = function checkSelfPromise(selfPromise) {
      return readModelSubscriptions.hasOwnProperty(subscriptionKey) && selfPromise === readModelSubscriptions[subscriptionKey].promise;
    };

    var selfPromise = Promise.resolve().then(_asyncToGenerator( /*#__PURE__*/regeneratorRuntime.mark(function _callee2() {
      var socketId, response, _ref5, result, timeToLive, serialId;

      return regeneratorRuntime.wrap(function _callee2$(_context2) {
        while (1) {
          switch (_context2.prev = _context2.next) {
            case 0:
              _context2.prev = 0;

              if (checkSelfPromise(selfPromise)) {
                _context2.next = 3;
                break;
              }

              return _context2.abrupt('return');

            case 3:
              _context2.next = 5;
              return subscribeAdapter.getClientId();

            case 5:
              socketId = _context2.sent;

              readModelSubscriptions[subscriptionKey].socketId = socketId;

              if (checkSelfPromise(selfPromise)) {
                _context2.next = 9;
                break;
              }

              return _context2.abrupt('return');

            case 9:
              _context2.next = 11;
              return orderedFetch((0, _util.getRootableUrl)('/api/query/' + readModelName + '/' + resolverName), {
                method: 'POST',
                headers: { 'Content-Type': 'application/json' },
                credentials: 'same-origin',
                body: JSON.stringify({
                  isReactive: isReactive,
                  socketId: socketId,
                  variables: variables
                })
              });

            case 11:
              response = _context2.sent;

              if (response.ok) {
                _context2.next = 14;
                break;
              }

              throw new Error();

            case 14:
              _context2.next = 16;
              return response.json();

            case 16:
              _ref5 = _context2.sent;
              result = _ref5.result;
              timeToLive = _ref5.timeToLive;
              serialId = _ref5.serialId;

              if (checkSelfPromise(selfPromise)) {
                _context2.next = 22;
                break;
              }

              return _context2.abrupt('return');

            case 22:
              store.dispatch(_actions2.default.loadReadmodelInitialState(readModelName, resolverName, result, serialId));

              if (isReactive) {
                _context2.next = 25;
                break;
              }

              return _context2.abrupt('return');

            case 25:
              _context2.next = 27;
              return (0, _util.delay)(timeToLive);

            case 27:
              fetchReadModel();
              _context2.next = 35;
              break;

            case 30:
              _context2.prev = 30;
              _context2.t0 = _context2['catch'](0);
              _context2.next = 34;
              return (0, _util.delay)(1000);

            case 34:
              fetchReadModel();

            case 35:
            case 'end':
              return _context2.stop();
          }
        }
      }, _callee2, _this, [[0, 30]]);
    })));

    readModelSubscriptions[subscriptionKey].promise = selfPromise;
  };

  readModelSubscriptions[subscriptionKey] = {
    promise: Promise.resolve(),
    refresh: fetchReadModel,
    socketId: null
  };

  fetchReadModel();
}

function unsubscribeReadmodel(store, readModelSubscriptions, orderedFetch, action) {
  var readModelName = action.readModelName,
      resolverName = action.resolverName;

  var subscriptionKey = readModelName + ':' + resolverName;
  if (!readModelSubscriptions.hasOwnProperty(subscriptionKey)) return;

  var socketId = readModelSubscriptions[subscriptionKey].socketId;
  delete readModelSubscriptions[subscriptionKey];

  store.dispatch(_actions2.default.dropReadmodelState(readModelName, resolverName));

  if (!socketId || socketId.constructor !== String) return;

  orderedFetch((0, _util.getRootableUrl)('/api/query/' + readModelName + '/' + resolverName + '/' + socketId), {
    method: 'DELETE',
    credentials: 'same-origin'
  }).catch(function (error) {
    return null;
  });
}

var isClient = typeof window !== 'undefined';

function createOrderedFetch() {
  var _this2 = this;

  var orderedFetchPromise = Promise.resolve();
  if (!isClient) {
    return function () {
      return Promise.reject('Ordered fetch can be used only on client side');
    };
  }

  var orderedFetch = function orderedFetch(url, options) {
    return new Promise(function (resolveResult) {
      orderedFetchPromise = orderedFetchPromise.then(_asyncToGenerator( /*#__PURE__*/regeneratorRuntime.mark(function _callee3() {
        return regeneratorRuntime.wrap(function _callee3$(_context3) {
          while (1) {
            switch (_context3.prev = _context3.next) {
              case 0:
                if (!true) {
                  _context3.next = 15;
                  break;
                }

                _context3.prev = 1;
                _context3.t0 = resolveResult;
                _context3.next = 5;
                return fetch(url, options);

              case 5:
                _context3.t1 = _context3.sent;
                return _context3.abrupt('return', (0, _context3.t0)(_context3.t1));

              case 9:
                _context3.prev = 9;
                _context3.t2 = _context3['catch'](1);
                _context3.next = 13;
                return new Promise(function (resolve) {
                  return setTimeout(resolve, 1000);
                });

              case 13:
                _context3.next = 0;
                break;

              case 15:
              case 'end':
                return _context3.stop();
            }
          }
        }, _callee3, _this2, [[1, 9]]);
      })));
    });
  };

  return orderedFetch;
}

var mockSubscribeAdapter = exports.mockSubscribeAdapter = {
  onEvent: function onEvent() {},
  onDisconnect: function onDisconnect() {},
  setSubscription: function setSubscription() {},
  getClientId: function getClientId() {
    return Promise.resolve('0');
  }
};

function createResolveMiddleware(_ref7) {
  var _ref7$viewModels = _ref7.viewModels,
      viewModels = _ref7$viewModels === undefined ? [] : _ref7$viewModels,
      _ref7$subscribeAdapte = _ref7.subscribeAdapter,
      subscribeAdapter = _ref7$subscribeAdapte === undefined ? _subscribe_adapter2.default : _ref7$subscribeAdapte;

  var subscribers = {
    viewModels: {},
    aggregateIds: {}
  };

  var requests = {};
  var loading = viewModels.reduce(function (acc, _ref8) {
    var name = _ref8.name;

    acc[name] = {};
    return acc;
  }, {});

  var readModelSubscriptions = {};

  var orderedFetch = createOrderedFetch();

  return function (store) {
    Object.defineProperty(store.getState, 'isLoadingViewModel', {
      enumerable: false,
      configurable: false,
      writable: false,
      value: function value(viewModelName, aggregateId) {
        return !!loading[viewModelName][aggregateId];
      }
    });

    store.dispatch(_actions2.default.provideViewModels(viewModels));

    var adapter = isClient ? subscribeAdapter() : mockSubscribeAdapter;
    adapter.onEvent(function (event) {
      return store.dispatch(event);
    });
    adapter.onDisconnect(function (error) {
      store.dispatch(_actions2.default.disconnect(error));

      Object.keys(readModelSubscriptions).forEach(function (subscriptionKey) {
        readModelSubscriptions[subscriptionKey].refresh();
      });
    });

    return function (next) {
      return function (action) {
        switch (action.type) {
          case _action_types.SUBSCRIBE_VIEWMODEL:
            {
              var viewModelName = action.viewModelName,
                  aggregateId = action.aggregateId;

              loading[viewModelName][aggregateId] = true;

              if (isClient) {
                subscribeViewmodel(store, adapter, viewModels, subscribers, requests, action).catch(function (error) {
                  return setTimeout(function () {
                    // eslint-disable-next-line no-console
                    console.error(error);
                    store.dispatch(action);
                  }, REFRESH_TIMEOUT);
                });
              }

              break;
            }
          case _action_types.UNSUBSCRIBE_VIEWMODEL:
            {
              var _viewModelName = action.viewModelName,
                  _aggregateId = action.aggregateId;

              delete loading[_viewModelName][_aggregateId];

              if (isClient) {
                unsubscribeViewmodel(store, adapter, viewModels, subscribers, requests, action);
              }

              break;
            }
          case _action_types.SUBSCRIBE_READMODEL:
            {
              if (isClient) {
                subscribeReadmodel(store, readModelSubscriptions, adapter, orderedFetch, action);
              }

              break;
            }
          case _action_types.UNSUBSCRIBE_READMODEL:
            {
              if (isClient) {
                unsubscribeReadmodel(store, readModelSubscriptions, orderedFetch, action);
              }

              break;
            }
          case _action_types.SEND_COMMAND:
            {
              if (isClient) {
                (0, _send_command2.default)(store, action);
              }

              break;
            }
          default:
        }

        return next(action);
      };
    };
  };
}

exports.default = createResolveMiddleware;